package thegame.client;

import java.awt.*;
import javax.swing.*;


/**
 * The game square. Can be clicked, circled, crossed, enabled and disabled.
 * @author Erki Suurjaak
 * @version 0.99 (2000/08/28)
 */
public class GameSquare
  extends JButton 
{
  /** Is true if the square is crossed, is false otherwise. */
  protected boolean
  isCrossed = false;

  /** Is true if the square is circled, is false otherwise. */
  protected boolean
  isCircled = false;

  /** Is true if the square has been circled or crossed and is part of a winning row. Is used in setHighlighted(boolean) - a winning square is not unhighlighted. */
  protected boolean
  isWinningSquare = false;

  /** The position of the square on the gamefield. */
  protected int[]
  position = {-1, -1};

  public static final String 
  DEFAULT_ICON_PATH = "images/defaultIcon.gif",
  PRESSED_ICON_PATH = "images/pressedIcon.gif",
  CIRCLED_ICON_PATH = "images/circledIcon.gif",
  CROSSED_ICON_PATH = "images/crossedIcon.gif",
  CLOSED_ICON_PATH = "images/closedIcon.gif",
  HIGHLIGHTED_CIRCLED_ICON_PATH = "images/highlightedCircledIcon.gif",
  HIGHLIGHTED_CROSSED_ICON_PATH = "images/highlightedCrossedIcon.gif",
  HIGHLIGHTED_CIRCLED_WINNING_ICON_PATH = "images/highlightedCircledWinningIcon.gif",
  HIGHLIGHTED_CROSSED_WINNING_ICON_PATH = "images/highlightedCrossedWinningIcon.gif";

  public static final ImageIcon
  DEFAULT_ICON = GameClient.getImageIcon(DEFAULT_ICON_PATH),
  PRESSED_ICON = GameClient.getImageIcon(PRESSED_ICON_PATH),
  CIRCLED_ICON = GameClient.getImageIcon(CIRCLED_ICON_PATH),
  CROSSED_ICON = GameClient.getImageIcon(CROSSED_ICON_PATH),
  CLOSED_ICON = GameClient.getImageIcon(CLOSED_ICON_PATH),
  HIGHLIGHTED_CIRCLED_ICON = GameClient.getImageIcon(HIGHLIGHTED_CIRCLED_ICON_PATH),
  HIGHLIGHTED_CROSSED_ICON = GameClient.getImageIcon(HIGHLIGHTED_CROSSED_ICON_PATH),
  HIGHLIGHTED_CIRCLED_WINNING_ICON = GameClient.getImageIcon(HIGHLIGHTED_CIRCLED_WINNING_ICON_PATH),
  HIGHLIGHTED_CROSSED_WINNING_ICON = GameClient.getImageIcon(HIGHLIGHTED_CROSSED_WINNING_ICON_PATH);


  /**
   * Toggles the highlighted state of the square.
   * @param highlighted if true, then the square will be highlighted; otherwise the square will be unhighlighted
   */
  public void
  setHighlighted(final boolean highlighted)
    {
      if (isCircled)
        {
          if (highlighted)
            setDisabledIcon(HIGHLIGHTED_CIRCLED_ICON);
          else
            {
              if (isWinningSquare)
                setDisabledIcon(HIGHLIGHTED_CIRCLED_WINNING_ICON);
              else
                setDisabledIcon(CIRCLED_ICON);
            }
        }
      else if (isCrossed)
        {
          if (highlighted)
            setDisabledIcon(HIGHLIGHTED_CROSSED_ICON);
          else
            {
              if (isWinningSquare)
                setDisabledIcon(HIGHLIGHTED_CROSSED_WINNING_ICON);
              else
                setDisabledIcon(CROSSED_ICON);
            }
        }
    }


  /**
   * Resets the square - sets it to be empty. 
   */
  public void
  reset(final boolean enabled)
    {
      isCrossed = isCircled = isWinningSquare = false;
      setIcon(DEFAULT_ICON);
      if (!isEnabled())
        super.setEnabled(true);
      setIcon(DEFAULT_ICON);
      if (enabled)
        setPressedIcon(PRESSED_ICON);
      else
        setPressedIcon(DEFAULT_ICON);
      return;
    }


  /**
   * Toggles the permanently disabled state of the square.
   */
  public void
  setClosed()
    {
      setIcon(CLOSED_ICON);
      setPressedIcon(CLOSED_ICON);
    }


  /**
   * Enables or disables user input. Does not disable the square, but modifies its behaviour to mimic a disabled square.
   * @param enabled true if the square can receive user input; false otherwise
   */
  public void
  setEnabled(final boolean enabled)
    {
      if (enabled)
        setPressedIcon(PRESSED_ICON);
      else
        setPressedIcon(DEFAULT_ICON);
    }


  /**
   * Returns the position property
   * @return the position property
   */
  public int[]
  getPosition()
    {
      return position;
    }


  /**
   * Sets the square part of a winning row. Highlights the square with a winning square icon.
   */
  public void
  setWinningSquare()
    {
      isWinningSquare = true;
      if (isCrossed)
        setDisabledIcon(HIGHLIGHTED_CROSSED_WINNING_ICON);
      else if (isCircled)
        setDisabledIcon(HIGHLIGHTED_CIRCLED_WINNING_ICON);
    }


  /**
   * Returns the isCrossed property.
   * @return the isCrossed property
   */
  public boolean
  isCrossed()
    {
      return isCrossed;
    }


  /**
   * Returns the isCircled property.
   * @return the isCircled property
   */
  public boolean
  isCircled()
    {
      return isCircled;
    }


  /**
   * Returns true if the square is empty (neither crossed nor circled); false otherwise.
   * @return true if the square is empty; false otherwise
   */
  public boolean
  isEmpty()
    {
      return !(isCircled || isCrossed); 
    }


  /**
   * Sets the square circled and disabled.
   */
  public void
  setCircled()
    {
      isCircled = true;
      setDisabledIcon(CIRCLED_ICON);
      super.setEnabled(false);
    }

  /**
   * Sets the square crossed and disabled.
   */
  public void
  setCrossed()
    {
      isCrossed = true;
      setDisabledIcon(CROSSED_ICON);
      super.setEnabled(false);
    }


  /**
   * Returns false to disable the square to receive keyboard focus.
   * @return false
   */
  public boolean
  isFocusTraversable()
    {
      return false;
    }


  /**
   * Constructs a GameSquare.
   * @param position the position of the square on the gamefield
   */
  public
  GameSquare(final int[] position)
    {
      System.arraycopy(position, 0, this.position, 0, 2);
      setContentAreaFilled(false);
      setFocusPainted(false);
      setMargin(new Insets(0, 0, 0, 0));
      setBorder(null);
      setIcon(DEFAULT_ICON);
      setPressedIcon(PRESSED_ICON);
      return;
    }


}
