package thegame.shared;

import java.io.*;
import java.net.*;
import java.util.*;


/**
 * Defines methods and data members used by GameClientConnection and GameServerConnection.
 * @author Erki Suurjaak
 * @version 0.99 (2000/08/28)
 */
public abstract class GameConnection
  extends Thread
  implements GameConnectionConstants
{
  /** The socket used. */
  protected Socket
  socket;

  /** Used for reading from the socket. */
  protected BufferedReader
  in;

  /** Used for writing to the socket. */
  protected PrintWriter
  out;

  /** Is true if the thread is running; is false otherwise. The inner loop in run() exits when this becomes false. */
  protected boolean
  isRunning = true;

  /**
   * Constructs a GameConnection on the specified socket.
   * @param socket the socket used; must be already created
   * @throws IOException if an I/O error occurs when creating the I/O streams
   */
  public
  GameConnection(Socket socket)
  throws IOException
    {
      this.socket = socket;
      in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
      out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())),true);
    }


  /**
   * Sends a string.
   * @param str the data to be sent
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final String str)
  throws IOException
    {
      out.println(str);
    }


  /**
   * Sends a message with id as the type identifier of the message.
   * @param id the type identifier of the message
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final int id)
  throws IOException
    {
      out.println(id);
    }


  /**
   * Sends a message with id as the message type identifier and body as the body of the message.
   * @param id the type identifier of the message
   * @param body the body of the message
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final int id, final String body)
  throws IOException
    {
      out.println(id + DELIMITER + body);
    }


  /**
   * Sends a message with id as the message type identifier and body as the body of the message.
   * @param id the type identifier of the message
   * @param body the body of the message
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final int id, final boolean body)
  throws IOException
    {
      out.println(id + DELIMITER + body);
    }


  /**
   * Sends a message with id as the type identifier of the message and body as the body of the message.
   * @param id the type identifier of the message
   * @param body the body of the message
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final int id, final int[] body)
  throws IOException
    {
      String str = String.valueOf(id) + DELIMITER;
      for (int i = 0; i < body.length; i++)
        {
          str += String.valueOf(body[i]);
          str += DELIMITER;
        }
      out.println(str);
    }


  /**
   * Sends a message with id as the type identifier of the message and body as the body of the message.
   * @param id the type identifier of the message
   * @param body the body of the message
   * @throws IOException if an I/O error occurs
   */
  public void
  send(final int id, final int[][] body)
  throws IOException
    {
      String str = String.valueOf(id) + DELIMITER;
      for (int i = 0; i < body.length; i++)
        {
          for (int j = 0; j < 2; j++)
            {
              str += String.valueOf(body[i][j]);
              str += DELIMITER;
            }
        }
      out.println(str);
    }


  /**
   * Returns the method arguments in a form that is ready to be sent as a message.
   * @param id the type identifier of the message
   * @param body the body of the message
   * @return the method arguments in a form that is ready to be sent as a message
   */
  public static String
  getReadyToSendForm(final int id, final String[] body)
    {
      String str = String.valueOf(id);
      if (body != null)
        {
          if (body.length > 0)
            str += DELIMITER;
          for (int i = 0; i < body.length; i++)
            str += body[i] + DELIMITER;
        }
      return str;
    }


  /**
   * Returns the body of the message, or null if the message has no body.
   * @param message the message to be parsed
   * @return the body of the message or null if the message has no body
   */
  public static String
  getMessageBody(final String message)
    {
      try
        {
          if (message.indexOf(DELIMITER) == -1)
            return null;
          return message.substring(message.indexOf(DELIMITER) + DELIMITER.length());
        }
      catch (Exception e)
        {
          return null;
        }
    }


  /**
   * Returns the type identifier of the message.
   * @param message the message to be parsed
   * @return the type identifier of the message
   */
  public static int
  getMessageID(final String message)
    {
      try
        {
          if (message.indexOf(DELIMITER) != -1)
            return Integer.valueOf(message.substring(0, message.indexOf(DELIMITER))).intValue();
          else
            return Integer.valueOf(message).intValue();
        }
      catch (Exception e)
        {
          return -1;
        }
    }


  /**
   * Returns the contents of the message in the form of int[x][2] (where x is undefined), or null if the message has no body.
   * @param message the message to be parsed
   * @return the contents of the message or null if the message has no body
   */
  public static int[][]
  getMessageContentsAs2DIntArray(final String message)
    {
      String body = getMessageBody(message);
      if (body == null)
        return null;
      ArrayList contents = new ArrayList();
      while (true)
        {
          try
            {
              int[] pair = new int[2];
              for (int i = 0; i < pair.length; i++)
                {
                  pair[i] = Integer.valueOf(body.substring(0, body.indexOf(DELIMITER))).intValue();
                  body = body.substring(body.indexOf(DELIMITER) + DELIMITER.length());
                }
              contents.add(pair);
            }
          catch (IndexOutOfBoundsException e)
            {
              int[][] pairs = new int[contents.size()][2];
              contents.toArray(pairs);
              return pairs;
            }
        }
    }


  /**
   * Returns the contents of the message as int[2], or null if the message has no body.
   * @param message the message to be parsed
   * @return the contents of the message as int[2] or null if the message has no body
   */
  public static int[]
  getMessageContentsAs1DIntArray(final String message)
    {
      String body = getMessageBody(message);
      if (body == null)
        return null;
      int[] pair = new int[2];
      try
        {
          for (int i = 0; i < pair.length; i++)
            {
              pair[i] = Integer.valueOf(body.substring(0, body.indexOf(DELIMITER))).intValue();
              body = body.substring(body.indexOf(DELIMITER) + DELIMITER.length());
            }
        }
      catch (IndexOutOfBoundsException e)
        {
          return null;
        }
      return pair;
    }


  /**
   * Returns the contents of the message as a String array, or null if the message has no body.
   * @param message the message to be parsed
   * @return the contents of the message or null if the message has no body
   */
  public static String[]
  getMessageContentsAs1DStringArray(final String message)
    {
      String body = getMessageBody(message);
      if (body == null)
        return null;
      ArrayList contents = new ArrayList();
      while (true)
        {
          try
            {
              String string = body.substring(0, body.indexOf(DELIMITER));
              body = body.substring(body.indexOf(DELIMITER) + DELIMITER.length());
              contents.add(string);
            }
          catch (IndexOutOfBoundsException e)
            {
              return (String[])contents.toArray(new String[1]);
            }
        }
    }


  /**
   * Returns the contents of the message as a boolean value.
   * @param message the message to be parsed
   * @return the contents of the message as a boolean value
   */
  public static boolean
  getMessageContentsAsBoolean(final String message)
    {
      return Boolean.valueOf(getMessageBody(message)).booleanValue();
    }


  /**
   * Sets the isRunning property.
   * @param running false if the GameConnection thread is to stop; true otherwise
   */
  public void
  setRunning(final boolean running)
    {
      isRunning = running;
    }


  /**
   * Closes the socket.
   */
  public void
  close()
    {
      try
        {
          socket.close();
          socket = null;
        }
      catch (IOException e)
        {
        }
    }


}
